import _extends from "@babel/runtime/helpers/esm/extends";
import { MASK, valueChecker, getValueType } from '../config';
import PIN_INPUT_ACTION_TYPES from './actionTypes';
import { debounce } from '@s-ui/js/lib/function';
export var getInitialPinInputReducerState = function getInitialPinInputReducerState(_temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      _ref$defaultValue = _ref.defaultValue,
      defaultValue = _ref$defaultValue === void 0 ? '' : _ref$defaultValue,
      _ref$disabled = _ref.disabled,
      disabled = _ref$disabled === void 0 ? false : _ref$disabled,
      _ref$mask = _ref.mask,
      mask = _ref$mask === void 0 ? MASK.NUMBER : _ref$mask,
      value = _ref.value;

  var valueType = getValueType({
    value: value,
    defaultValue: defaultValue
  });
  var arrayValue = typeof value === 'string' ? value.split('') : value;
  var arrayDefaultValue = typeof defaultValue === 'string' ? defaultValue.split('') : defaultValue;
  var innerValue = value !== undefined ? arrayValue : arrayDefaultValue;
  innerValue = valueChecker({
    mask: mask,
    length: innerValue.length
  })(innerValue.filter(Boolean).join('')) ? innerValue : [];
  return {
    valueType: valueType,
    focusPosition: 0,
    mask: mask,
    innerValue: innerValue,
    checker: valueChecker({
      mask: mask
    }),
    disabled: disabled,
    elements: []
  };
};

var focus = function focus(element) {
  return setTimeout(function () {
    return element.focus();
  }, 0);
};

var debouncedFocus = debounce(focus, 20);

var focusElement = function focusElement(element) {
  debouncedFocus(element);
};

var onChangeHandler = function onChangeHandler(onChange) {
  return function (event, state) {
    var innerValue = state.innerValue,
        focusPosition = state.focusPosition,
        valueType = state.valueType;
    typeof onChange === 'function' && onChange(event, {
      value: valueType === 'string' ? innerValue.filter(Boolean).join('') : innerValue,
      key: innerValue[focusPosition],
      index: focusPosition,
      innerValue: innerValue
    });
  };
};

export var pinInputReducer = function pinInputReducer(state, _ref2) {
  var _payload$innerValue;

  var actionType = _ref2.actionType,
      payload = _ref2.payload;
  var nextState = Object.assign({}, state);
  var checker = state.checker,
      innerValue = state.innerValue,
      focusPosition = state.focusPosition,
      elements = state.elements;
  var disabled = payload.disabled,
      mask = payload.mask,
      node = payload.node,
      _payload$event = payload.event,
      event = _payload$event === void 0 ? {} : _payload$event;
  var key = event.key,
      shiftKey = event.shiftKey;
  var onChange = onChangeHandler(payload.onChange);
  var isValidPayloadInnerValue;
  var newChecker;
  var isInvalid;
  var newIndex;
  var position;

  switch (actionType) {
    case PIN_INPUT_ACTION_TYPES.SET_PIN_INPUT_DISABLED:
      nextState = _extends({}, state, {
        disabled: disabled
      });
      break;

    case PIN_INPUT_ACTION_TYPES.SET_PIN_INPUT_VALUE:
      isValidPayloadInnerValue = valueChecker({
        mask: state.mask,
        length: payload.innerValue.length
      })(payload.innerValue.filter(Boolean).join(''));

      if (isValidPayloadInnerValue && !((innerValue == null ? void 0 : innerValue.length) === (payload == null ? void 0 : (_payload$innerValue = payload.innerValue) == null ? void 0 : _payload$innerValue.length) && innerValue.every(function (value, index) {
        return value === payload.innerValue[index];
      }))) {
        nextState = _extends({}, state, {
          innerValue: [].concat(payload.innerValue)
        });
      }

      break;

    case PIN_INPUT_ACTION_TYPES.SET_PIN_INPUT_MASK:
      newChecker = valueChecker({
        mask: mask
      });
      isInvalid = innerValue.find(function (value) {
        return !newChecker(value);
      });
      nextState = _extends({}, state, {
        checker: newChecker,
        mask: mask,
        innerValue: isInvalid ? [] : innerValue
      });
      break;

    case PIN_INPUT_ACTION_TYPES.SET_PIN_INPUT_KEY:
      if (key.length > 1) {
        switch (key) {
          case 'ArrowUp':
          case 'ArrowDown':
            focusElement(elements[focusPosition]);
            break;

          case 'ArrowLeft':
            nextState = _extends({}, state, {
              focusPosition: elements[focusPosition - 1] ? focusPosition - 1 : focusPosition
            });
            focusElement(elements[nextState.focusPosition]);
            break;

          case 'ArrowRight':
            nextState = _extends({}, state, {
              focusPosition: elements[focusPosition + 1] ? focusPosition + 1 : focusPosition
            });
            focusElement(elements[nextState.focusPosition]);
            break;

          case 'Backspace':
            innerValue[focusPosition] = undefined;
            nextState = _extends({}, state, {
              innerValue: [].concat(innerValue),
              focusPosition: elements[focusPosition - 1] ? focusPosition - 1 : focusPosition
            });
            onChange(event, nextState);
            break;

          case 'Delete':
            innerValue[focusPosition] = undefined;
            nextState = _extends({}, state, {
              innerValue: [].concat(innerValue)
            });
            onChange(event, nextState);
            break;

          case 'Tab':
            newIndex = shiftKey ? focusPosition - 1 : focusPosition + 1;
            nextState = _extends({}, state, {
              focusPosition: elements[newIndex] ? newIndex : focusPosition
            });

            if (newIndex < elements.length && newIndex > 0) {
              focusElement(elements[nextState.focusPosition]);
            }

            break;

          case 'Meta':
          default:
            break;
        }
      } else if (checker(key)) {
        var isKeyChange = innerValue[focusPosition] !== key;

        if (isKeyChange) {
          innerValue[focusPosition] = key;
        }

        nextState = _extends({}, state, {
          innerValue: isKeyChange ? [].concat(innerValue) : innerValue,
          focusPosition: elements[focusPosition + 1] ? focusPosition + 1 : focusPosition
        });

        if (isKeyChange) {
          onChange(event, nextState);
        }

        focusElement(elements[nextState.focusPosition]);
      } else {
        focusElement(elements[focusPosition]);
      }

      break;

    case PIN_INPUT_ACTION_TYPES.SET_PIN_INPUT_FOCUS:
      position = elements[payload.focusPosition] ? payload.focusPosition : focusPosition;
      nextState = _extends({}, state, {
        focusPosition: position
      });
      focusElement(elements[position]);
      break;

    case PIN_INPUT_ACTION_TYPES.SET_PIN_INPUT_ELEMENT:
      if (!elements.includes(node)) {
        elements[elements.length] = node;
        nextState = _extends({}, state, {
          elements: [].concat(elements)
        });
      }

      break;

    case PIN_INPUT_ACTION_TYPES.REMOVE_PIN_INPUT_ELEMENT:
      nextState = _extends({}, state, {
        elements: elements.filter(function (element) {
          return node !== element;
        })
      });
      break;

    default:
      break;
  }

  return nextState;
};